/*
 * contiki-conf.h
 *
 * Copyright (c) 2012, ROSAND Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Contiki operating system.
 *
 */

#ifndef __CONTIKI_CONF_H__
#define __CONTIKI_CONF_H__

/* Platform name, type, and MCU clock rate */
#define PLATFORM_NAME  "ROSAND R-Mote p8"
#define PLATFORM_TYPE  RCB_B
#define F_CPU          8000000UL

#include <stdint.h>

/* The AVR tick interrupt usually is done with an 8 bit counter around 128 Hz.
 * 125 Hz needs slightly more overhead during the interrupt, as does a 32 bit
 * clock_time_t.
 */
/* Clock ticks per second */
#define CLOCK_CONF_SECOND 128

/* 16 bit counter overflows every ~10 minutes */
typedef unsigned short clock_time_t;
#define CLOCK_LT(a,b)  ((signed short)((a)-(b)) < 0)

/* These routines are not part of the contiki core but can be enabled in cpu/avr/clock.c */
void clock_delay_msec(uint16_t howlong);
void clock_adjust_ticks(clock_time_t howmany);

/* COM port to be used for SLIP connection. Not needed on R-Mote p8. */
#define SLIP_PORT RS232_PORT_1

/* Starting address for code received via the codeprop facility. Not tested */
typedef unsigned long off_t;

/*
 * Sensors
 */
#ifndef ENV_SENSOR_CONF_ON
#define ENV_SENSOR_CONF_ON              0  /* Environment Sensor */
#endif

#ifndef GPS_SENSOR_CONF_ON
#define GPS_SENSOR_CONF_ON              0  /* GPS Sensor */
#endif

/* More extensive stats */
#define ENERGEST_CONF_ON                0

/* Packet statistics */
typedef unsigned short uip_stats_t;
#define UIP_STATISTICS                  0

/* Enable autostart process */
#define AUTOSTART_ENABLE                1

/* Network setup */

#if UIP_CONF_IPV6
#define RIMEADDR_CONF_SIZE              8
#define UIP_CONF_ICMP6                  1
#define UIP_CONF_UDP                    1

#define UIP_CONF_IPV6_CHECKS            1
#define UIP_CONF_IPV6_QUEUE_PKT         0
#define UIP_CONF_IPV6_REASSEMBLY        0

#define NETSTACK_CONF_NETWORK           sicslowpan_driver
#define SICSLOWPAN_CONF_FRAG            1
#define SICSLOWPAN_CONF_MAXAGE          8
#define SICSLOWPAN_CONF_COMPRESSION     SICSLOWPAN_COMPRESSION_HC06
#define SICSLOWPAN_CONF_MAX_ADDR_CONTEXTS       0
#define SICSLOWPAN_CONF_ADDR_CONTEXT_0 { \
  addr_contexts[0].prefix[0] = 0x20; \
  addr_contexts[0].prefix[1] = 0x01; \
  addr_contexts[0].prefix[2] = 0x0d; \
  addr_contexts[0].prefix[3] = 0xb8; \
  addr_contexts[0].prefix[4] = 0x00; \
  addr_contexts[0].prefix[5] = 0x01; \
}
#else
/* ip4 should build but is largely untested */
#define RIMEADDR_CONF_SIZE              2
#define NETSTACK_CONF_NETWORK           rime_driver
#endif /* UIP_CONF_IPV6 */

#define UIP_CONF_LL_802154              1
#define UIP_CONF_LLH_LEN                0

#define UIP_CONF_IP_FORWARD             0
#define UIP_CONF_FWCACHE_SIZE           0

#define UIP_CONF_UDP_CHECKSUMS          1
#define UIP_CONF_TCP_SPLIT              0
#define UIP_CONF_DHCP_LIGHT             0

#ifndef UIP_CONF_BUFFER_SIZE
#define UIP_CONF_BUFFER_SIZE            480
#endif /* UIP_CONF_BUFFER_SIZE */

#ifndef NETSTACK_CONF_MAC
#define NETSTACK_CONF_MAC               csma_driver
#endif /* NETSTACK_CONF_MAC */
#define MAC_CONF_CHANNEL_CHECK_RATE     8

#ifndef NETSTACK_CONF_RDC
#define NETSTACK_CONF_RDC               sicslowmac_driver
#endif /* NETSTACK_CONF_RDC */

#ifndef NETSTACK_CONF_RDC_CHANNEL_CHECK_RATE
#define NETSTACK_CONF_RDC_CHANNEL_CHECK_RATE 8
#endif /* NETSTACK_CONF_RDC_CHANNEL_CHECK_RATE */

#define NETSTACK_CONF_FRAMER            framer_802154
#define NETSTACK_CONF_RADIO             rmote_driver

/* RF Config */
#ifndef IEEE802154_CONF_PANID
#define IEEE802154_CONF_PANID           0xABCD
#endif /* IEEE802154_CONF_PANID */

#ifndef CHANNEL_802_15_4
#define CHANNEL_802_15_4                26
#endif /* CHANNEL_802_15_4 */

/* Allocate remaining RAM as desired */
/* 30 bytes per TCP connection */
/* 6LoWPAN does not do well with concurrent TCP streams, as new browser GETs collide with packets coming */
/* from previous GETs, causing decreased throughput, retransmissions, and timeouts. Increase to study this. */
/* ACKs to other ports become interleaved with computation-intensive GETs, so ACKs are particularly missed. */
/* Increasing the number of packet receive buffers in RAM helps to keep ACKs from being lost */
#define UIP_CONF_MAX_CONNECTIONS        4
/* 2 bytes per TCP listening port */
#define UIP_CONF_MAX_LISTENPORTS        2
/* 25 bytes per UDP connection */
#define UIP_CONF_UDP_CONNS              4
/* 211 bytes per queue buffer */
#define QUEUEBUF_CONF_NUM               6

/* Logging adds 200 bytes to program size */
#define LOG_CONF_ENABLED                0

/* ************************************************************************** */
//#pragma mark RPL Settings
/* ************************************************************************** */
#if UIP_CONF_RPL

#define UIP_CONF_ROUTER                 1
#define UIP_CONF_ND6_SEND_RA            0
#define UIP_CONF_ND6_REACHABLE_TIME     600000
#define UIP_CONF_ND6_RETRANS_TIMER      10000
#undef UIP_CONF_FWCACHE_SIZE
#define UIP_CONF_FWCACHE_SIZE           10
#define UIP_CONF_BROADCAST              1
#define UIP_ARCH_IPCHKSUM               1
#define UIP_CONF_PINGADDRCONF           0
#define UIP_CONF_LOGGING                0

#ifndef RPL_CONF_OF
#define RPL_CONF_OF rpl_of_etx
#endif

/* See uip-ds6.h */
#define UIP_CONF_DS6_NBR_NBU            20
#define UIP_CONF_DS6_DEFRT_NBU          20
#define UIP_CONF_DS6_PREFIX_NBU         20
#define UIP_CONF_DS6_ROUTE_NBU          20
#define UIP_CONF_DS6_ADDR_NBU           20
#define UIP_CONF_DS6_MADDR_NBU          0
#define UIP_CONF_DS6_AADDR_NBU          0

#else
#define UIP_CONF_ROUTER                 0
#endif /* RPL */

#define CCIF
#define CLIF

/* include the project config */
/* PROJECT_CONF_H might be defined in the project Makefile */
#ifdef PROJECT_CONF_H
#include PROJECT_CONF_H
#endif /* PROJECT_CONF_H */

#endif /* __CONTIKI_CONF_H__ */
