/*
 * rmote-sensors.c
 *
 * Copyright (c) 2012, ROSAND Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the Institute nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE INSTITUTE AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE INSTITUTE OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * This file is part of the Contiki operating system.
 *
 */

#include "sensors.h"
#include "dev/button-sensor.h"
#include "rmote-sensors.h"
#include "dev/env-process.h"
#include "dev/gps-process.h"
#include "dev/leds.h"

const struct sensors_sensor *sensors[] = {
#if ENV_SENSOR_ON
    &env_sensor,
#endif
#if GPS_SENSOR_ON
    &gps_sensor,
#endif
    0
};

unsigned char sensors_flags[(sizeof(sensors) / sizeof(struct sensors_sensor *))];

process_event_t rmote_event;

#define SENSOR_DATA_LENGTH  32

/*---------------------------------------------------------------------------*/
PROCESS(rmote_sensor_process, "R-Mote Sensors Process");
/*---------------------------------------------------------------------------*/

void
init_sensors(void)
{
  process_start(&sensors_process, NULL);
  process_start(&rmote_sensor_process, NULL);
#if ENV_SENSOR_ON
  env_sensor_handle.init();
#endif
#if GPS_SENSOR_ON
  gps_sensor_handle.init();
#endif
}

/*---------------------------------------------------------------------------*/

PROCESS_THREAD(rmote_sensor_process, ev, data)
{
  static uint8_t buf[SENSOR_DATA_LENGTH];
  uint8_t len;

  PROCESS_BEGIN();

  while(1) {
    PROCESS_WAIT_EVENT_UNTIL(ev == sensors_event);
    len = 0;
    if(data == &env_sensor) {
      len = env_sensor_handle.handle(buf);
    } else if(data == &gps_sensor) {
      len = gps_sensor_handle.handle(buf);
    }
    if (len) {
      process_post(PROCESS_BROADCAST, rmote_event, buf);
    }
  }
  PROCESS_END();
}

/*---------------------------------------------------------------------------*/
